package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.AttributModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturnException;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link AttributeModifier}. An {@link Attribute} with this modifier will be generated as
 * <code>transient</code>.
 */
public final class AttributeModifierTransient extends AttributeModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1005956425849099876L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 32409324;
	
	/**
	 * Private Constructor of {@link AttributeModifierTransient}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private AttributeModifierTransient(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link AttributeModifierTransient}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The AttributeModifierTransient-Object.
	 */
	public static AttributeModifierTransient create(final Token firstToken, final Token lastToken) {
		return new AttributeModifierTransient(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link AttributeModifierTransient}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The AttributeModifierTransient-Object.
	 */
	public static AttributeModifierTransient create(final Token firstToken) {
		return new AttributeModifierTransient(firstToken, null);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof AttributeModifierTransient;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_TRANSIENT;
	}
	
	@Override
	public boolean accept(final AttributModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final AttributeModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final AttributeModifierVisitorReturnException<X, Y> visitor) throws Y {
		return visitor.handle(this);
	}
}
