package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.type.InvalidState;
import de.fhdw.wtf.common.ast.visitor.ConstructorReferenceStateVisitor;
import de.fhdw.wtf.common.ast.visitor.ConstructorReferenceStateVisitorException;
import de.fhdw.wtf.common.ast.visitor.ConstructorReferenceStateVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.ConstructorReferenceStateVisitorReturnException;

/**
 * This class represents an invalid state of a {@link ConstructorReference}. That means, that the referencer did not
 * find a matching definition constructor for the applied constructor definition.
 */
public final class ConstructorInvalidState implements ConstructorReferenceState {
	
	/**
	 * the hashCode for this class, which is constant because only one InvalidState exist
	 */
	private final Integer HASHCODE_CONSTANT = 389856;
	
	/**
	 * Constructor of {@link InvalidState}.
	 */
	private ConstructorInvalidState() {
	}
	
	/**
	 * Creates a {@link ConstructorInvalidState}-Object.
	 * 
	 * @return The ConstructorInvalidState-Object
	 */
	public static ConstructorInvalidState create() {
		return new ConstructorInvalidState();
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof ConstructorInvalidState;
	}
	
	@Override
	public int hashCode() {
		return this.HASHCODE_CONSTANT;
	}
	
	@Override
	public String toString() {
		return ConstructorInvalidState.class.getSimpleName();
	}
	
	@Override
	public void accept(final ConstructorReferenceStateVisitor v) {
		v.handle(this);
	}
	
	@Override
	public <X> X accept(final ConstructorReferenceStateVisitorReturn<X> v) {
		return v.handle(this);
	}
	
	@Override
	public <Y extends Exception> void accept(final ConstructorReferenceStateVisitorException<Y> v) throws Y {
		v.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final ConstructorReferenceStateVisitorReturnException<X, Y> v) throws Y {
		return v.handle(this);
	}
}
