package de.fhdw.wtf.common.ast.type;

import java.util.Collection;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.visitor.ClassTypeVisitor;
import de.fhdw.wtf.common.ast.visitor.ClassTypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.ClassTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.ClassTypeVisitorReturnException;
import de.fhdw.wtf.common.token.Token;

/**
 * This class represents an Exception for code generation. Such a {@link ExceptionClassType} is always held by a
 * <Group>, can contain multiple {@link ClassModifier}s , multiple {@link Attribute}s, multiple {@link Operation}s and
 * extend other {@link ClassType}s.
 */
public class ExceptionClassType extends ClassType {
	
	/**
	 * Constructor of {@link ExceptionClassType}.
	 * 
	 * @param name
	 *            The name of the {@link ExceptionClassType}.
	 * @param modifiers
	 *            The modifiers of the {@link ExceptionClassType}.
	 * @param attributes
	 *            The attributes of the {@link ExceptionClassType}.
	 * @param superTypes
	 *            The super types of the {@link ExceptionClassType}.
	 * @param operations
	 *            The operations of the {@link ExceptionClassType}.
	 * @param constructors
	 *            The constructors called by {@link ExceptionClassType}.
	 * @param firstToken
	 *            firstToken
	 * @param subTypes
	 *            The sub types of the {@link ExceptionClassType}.
	 * @param lastToken
	 *            lastToken
	 */
	protected ExceptionClassType(final Name name,
			final Collection<ClassModifier> modifiers,
			final Collection<Attribute> attributes,
			final Collection<Type> superTypes,
			final Collection<Operation> operations,
			final Collection<Constructor> constructors,
			final Token firstToken,
			final Collection<ClassType> subTypes,
			final Token lastToken) {
		super(name, modifiers, attributes, superTypes, operations, constructors, firstToken, subTypes, lastToken);
		
	}
	
	/**
	 * Creates a {@link ExceptionClassType}-Object.
	 * 
	 * @param name
	 *            The name of the {@link ExceptionClassType}.
	 * @param modifiers
	 *            The modifiers of the {@link ExceptionClassType}.
	 * @param attributes
	 *            The attributes of the {@link ExceptionClassType}.
	 * @param superTypes
	 *            The super types of the {@link ExceptionClassType}.
	 * @param operations
	 *            The operations of the {@link ExceptionClassType}.
	 * @param superConstructors
	 *            The constructors called by {@link ExceptionClassType}.
	 * @param firstToken
	 *            firstToken
	 * @param subTypes
	 *            The sub types of the {@link ExceptionClassType}.
	 * @param lastToken
	 *            lastToken
	 * @return The {@link ExceptionClassType}-Object.
	 */
	public static ExceptionClassType create(final Name name,
			final Collection<ClassModifier> modifiers,
			final Collection<Attribute> attributes,
			final Collection<Type> superTypes,
			final Collection<Operation> operations,
			final Collection<Constructor> superConstructors,
			final Token firstToken,
			final Collection<ClassType> subTypes,
			final Token lastToken) {
		return new ExceptionClassType(name, modifiers, attributes, superTypes, operations, superConstructors,
				firstToken, subTypes, lastToken);
	}
	
	/**
	 * Creates a {@link ExceptionClassType}-Object.
	 * 
	 * @param name
	 *            The name of the {@link ExceptionClassType}.
	 * @param modifiers
	 *            The modifiers of the {@link ExceptionClassType}.
	 * @param attributes
	 *            The attributes of the {@link ExceptionClassType}.
	 * @param superTypes
	 *            The super types of the {@link ExceptionClassType}.
	 * @param operations
	 *            The operations of the {@link ExceptionClassType}.
	 * @param superConstructors
	 *            The constructors called by {@link ExceptionClassType}.
	 * @param firstToken
	 *            firstToken
	 * @param subTypes
	 *            The sub types of the {@link ExceptionClassType}.
	 * @return The {@link ExceptionClassType}-Object.
	 */
	public static ExceptionClassType create(final Name name,
			final Collection<ClassModifier> modifiers,
			final Collection<Attribute> attributes,
			final Collection<Type> superTypes,
			final Collection<Operation> operations,
			final Collection<Constructor> superConstructors,
			final Token firstToken,
			final Collection<ClassType> subTypes) {
		return new ExceptionClassType(name, modifiers, attributes, superTypes, operations, superConstructors,
				firstToken, subTypes, null);
	}
	
	@Override
	public void accept(final ClassTypeVisitor classTypeVisitor) {
		classTypeVisitor.handle(this);
	}
	
	@Override
	public <X> X accept(final ClassTypeVisitorReturn<X> classTypeVisitorReturn) {
		return classTypeVisitorReturn.handle(this);
	}
	
	@Override
	public <Y extends Exception> void accept(final ClassTypeVisitorException<Y> classTypeVisitorException) throws Y {
		classTypeVisitorException.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final ClassTypeVisitorReturnException<X, Y> classTypeVisitorReturnException)
			throws Y {
		return classTypeVisitorReturnException.handle(this);
	}
}
