package de.fhdw.wtf.common.ast.visitor;

import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ByReferenceState;
import de.fhdw.wtf.common.ast.type.InvalidState;

/**
 * Return-Exception-Visitor for {@link de.fhdw.wtf.common.ast.type.TypeProxyState}.
 * 
 * @param <X>
 *            return type of this visitor
 * @param <Y>
 *            ExceptionType
 */
public interface TypeProxyStateVisitorReturnException<X, Y extends Exception> {
	
	/**
	 * Handle operations for the type {@link ByNameState}.
	 * 
	 * @param byName
	 *            {@link ByNameState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(ByNameState byName) throws Y;
	
	/**
	 * Handle operations for the type {@link InvalidState}.
	 * 
	 * @param invalid
	 *            {@link InvalidState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(InvalidState invalid) throws Y;
	
	/**
	 * Handle operations for the type {@link ByReferenceState}.
	 * 
	 * @param byReference
	 *            {@link ByReferenceState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(ByReferenceState byReference) throws Y;
	
}
