package de.fhdw.wtf.common.exception.walker;

import de.fhdw.wtf.common.ast.SyntaxObject;
import de.fhdw.wtf.common.exception.editor.EditorMarkable;
import de.fhdw.wtf.common.exception.editor.EditorMarker;
import de.fhdw.wtf.common.token.Position;

/**
 * Represents the mismatch between a constructors name and its corresponding classes name.
 */
public abstract class EditorMarkableConstructorException extends TaskException implements EditorMarkable {
	
	/**
	 * The constructor referenced by this exception.
	 */
	private final SyntaxObject constructor;
	
	/**
	 * Instantiates a {@link EditorMarkableConstructorException}.
	 * 
	 * @param constructor
	 *            the constructor which has the mismatching name.
	 * @param message
	 *            the exception message.
	 */
	protected EditorMarkableConstructorException(final SyntaxObject constructor, final String message) {
		super(message);
		this.constructor = constructor;
	}
	
	@Override
	public Position getStartPosition() {
		return this.constructor.getFirstToken().getPosition();
	}
	
	@Override
	public String getDescription() {
		return this.getMessage();
	}
	
	@Override
	public Position getEndPos() {
		return this.constructor.getLastToken().getPosition();
	}
	
	@Override
	public EditorMarker getMarker() {
		return EditorMarker.INVALID_CONSTRUCTOR_MARKER;
	}
}
