package de.fhdw.wtf.common.exception.walker;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.exception.editor.EditorMarkable;
import de.fhdw.wtf.common.exception.editor.EditorMarker;
import de.fhdw.wtf.common.token.Position;

/**
 * Diese Fehlermeldung wird in SymmetricCheck.java geworfen, wenn zu einer symmetrischen Beziehung keine Gegenstelle
 * angegeben wurde.
 * 
 * @author hfw413hy
 *
 */
public class SymmetricCheckException extends TaskException implements EditorMarkable {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1608006835364421974L;
	
	private final Attribute attribute;
	
	/**
	 * Erzeugt eine Exception.
	 * 
	 * @param message
	 *            Die zugrunde liegende Fehlermeldung.
	 * @param attribute
	 *            Das Attribut dessen Gegenpart fehlt.
	 * @return Die Fehlermeldung.
	 */
	public static SymmetricCheckException create(final String message, final Attribute attribute) {
		return new SymmetricCheckException(message, attribute);
	}
	
	private SymmetricCheckException(final String message, final Attribute attribute) {
		super(message);
		this.attribute = attribute;
	}
	
	@Override
	public EditorMarker getMarker() {
		return EditorMarker.MISSING_SYMMETRIC_PART_MARKER;
	}
	
	@Override
	public String getDescription() {
		return this.getMessage();
	}
	
	@Override
	public Position getStartPosition() {
		return this.attribute.getFirstToken().getPosition();
	}
	
	@Override
	public Position getEndPos() {
		return this.attribute.getLastToken().getPosition();
	}
	
}
