package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the bracket close symbol ')' as a token.
 */
public final class BracketCloseToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 2261991508621064498L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 879234;
	
	/**
	 * Private constructor for {@link BracketOpenToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol ')' in the original input.
	 */
	private BracketCloseToken(final Position position) {
		super(position, SymbolConstants.BRACKETCLOSE);
	}
	
	/**
	 * Factory method for {@link BracketCloseToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static BracketCloseToken create(final Position position) {
		return new BracketCloseToken(position);
	}
	
	@Override
	public boolean isBracketCloseToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isBracketCloseToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.BRACKETCLOSE;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
