package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the pipe symbol '|' as a token.
 */
public final class PipeToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1650406547054110957L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 23624378;
	
	/**
	 * Private constructor for {@link PipeToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol '|' in the original input.
	 */
	private PipeToken(final Position position) {
		super(position, SymbolConstants.PIPE);
	}
	
	/**
	 * Factory method for {@link PipeToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static PipeToken create(final Position position) {
		return new PipeToken(position);
	}
	
	@Override
	public boolean isPipeToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isPipeToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.PIPE;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
