package de.fhdw.wtf.walker.tasks;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.ConstructorOrOperation;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.OperationModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.exception.walker.AbstractOperationsException;
import de.fhdw.wtf.common.exception.walker.TaskException;
import de.fhdw.wtf.common.task.TaskExecutor;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

/**
 * This Task check that only abstract classes has abstract oparations!
 * 
 */
public final class AbstractOperationsCheck extends SimpleWalkerTask {
	
	/**
	 * Constructor of {@link AbstractOperationsCheck}.
	 * 
	 * @param m
	 *            model
	 * @param taskmanager
	 *            taskmanager
	 */
	private AbstractOperationsCheck(final Model m, final TaskExecutor taskmanager) {
		super(m, taskmanager);
	}
	
	/**
	 * Creates a {@link AbstractOperationsCheck}-Object.
	 * 
	 * @param m
	 *            model
	 * @param taskmanager
	 *            taskmanager
	 * @return The {@link AbstractOperationsCheck}-Object.
	 */
	public static AbstractOperationsCheck create(final Model m, final TaskExecutor taskmanager) {
		return new AbstractOperationsCheck(m, taskmanager);
	}
	
	@Override
	public void handleGroup(final Group g) throws TaskException {
		// Nothing to do here
	}
	
	@Override
	public void handleClass(final ClassType c) throws TaskException {
		final boolean isAbstractClass = c.getModifiers().contains(ClassModifierAbstract.create(null));
		if (!isAbstractClass) {
			final OperationModifierAbstract operationModifierAbstract = OperationModifierAbstract.create(null);
			for (final Operation op : c.getOperations()) {
				if (op.getModifiers().contains(operationModifierAbstract)) {
					throw AbstractOperationsException.create(op.getFirstToken(), op.getLastToken());
				}
			}
		}
	}
	
	@Override
	public void handleAttribute(final Attribute a, final ClassType owner) throws TaskException {
		// Nothing to do here
	}
	
	@Override
	public void finalizeTask() throws TaskException {
		// Nothing to do here
	}
	
	@Override
	public void beginTask() throws TaskException {
		// Nothing to do here
	}
	
	@Override
	public void handleConstructorOrOperation(final ConstructorOrOperation coo, final ClassType owner)
			throws TaskException {
		// Nothing to do here.
		
	}
}
